@@ -9,7 +9,7 @@ import os
 from pathlib import Path
 from typing import Optional
 
-import sh.contrib
+from sh.contrib import git as _git
 
 
 def git(*args, index: Optional[Path] = None, **kwargs):
@@ -22,7 +22,7 @@ def git(*args, index: Optional[Path] = None, **kwargs):
     e = os.environ.copy()
     if index:
         e["GIT_INDEX_FILE"] = str(index)
-    return sh.contrib.git(*args, **kwargs, _env=e)
+    return _git(*args, **kwargs, _env=e)
 
 
-git.lfs = sh.contrib.git.lfs
+git.lfs = _git.lfs
